﻿// SpringyLetters.as
//
// Jim Bumgardner
//
// This is a higher-level class that provides the 'colored letters on a string'
// animation.  It uses the lowerlevel 'SpringSim' to accomplish the hard stuff.
//
// This class is mainly concerned with setting up the simulation and the initial arrangement
// of the letters, all of which happens in 'setupTitle' below.  Most of the interesting physics-related
// stuff happens in 'SpringSim', my lower-level spring-physics simulation.
//
// This object assumes the existence of a movie in the library called 'letter_mc' which
// is used to contruct letter tiles with.  It must contain a dynamic text field called 'letter_txt'
// with an embedded font.  The font won't rotate properly if it's not embedded.
//

#include "SpringSim.as"

function SpringyLetters(gravK, stiffK, frictionK, springTautness, 
						initTitle, anchorX, anchorY, stringWidth, firstLayer)
{
	if (firstLayer == undefined)
		firstLayer = 100;
	this.ss = new SpringSim(_root, gravK, stiffK, frictionK, true);
	this.springTautness = springTautness;
	this.nbrLetters = 0;
	this.firstDotLayer = firstLayer;
	this.anchorX = anchorX;	
	this.anchorY = anchorY;
	this.stringWidth = stringWidth;
	this.setupTitle(initTitle);
}

SpringyLetters.scaleToFit = true; // static props
SpringyLetters.rotateToFit = true;

// This function can rotation and scale of each letter to match position on spring-path.
//
MovieClip.prototype.letterAdjust = function()
{
	var mc1 = _root["dot"+(this.idx-1)];   // neighboring movieclips on the string of letters
	var mc2 = _root["dot"+(this.idx+1)];
	var dx = mc2._x - mc1._x;			   // the distance between those clips
	var dy = mc2._y - mc1._y;
	if (SpringyLetters.rotateToFit)
		this._rotation = Math.atan2(dy, dx)*180/Math.PI;
	else
		this._rotation = 0;
	if (SpringyLetters.scaleToFit)
		this._xscale = this._yscale = (Math.sqrt(dx*dx+dy*dy)/2)*100/this.orig_width;
	else
		this._xscale = this._yscale = 100;
}

// This is used to add anchor points for the string - each string begins and ends
// with an invisible movieclip that is fixed into place.
//

SpringyLetters.prototype.addAnchor = function(x,y,layer)
{
	var mc = _root.createEmptyMovieClip("dot" + this.nbrLetters, layer);
	mc._x = x;
	mc._y = y;
	mc.idx = this.nbrLetters;
	this.nbrLetters++;
	mc.setDotProps(true,false);
	return mc;
}

// This adds a colored letter to the string
//
SpringyLetters.prototype.addLetter = function(x,y,letter,clr,layer)
{
	var mc = _root.attachMovie("letter_mc", "dot" + this.nbrLetters, layer);
	mc._x = x;
	mc._y = y;
	mc.letter_txt.autoSize = true;
	mc.letter_txt.text = letter;
	mc.letter_txt.textColor = clr;
	mc.orig_width = mc.letter_txt.textWidth;
	mc.onEnterFrame = mc.letterAdjust;
	mc.idx = this.nbrLetters;
	this.nbrLetters++;
	mc.drawDot();
	return mc;
}

// Get a purty/ugly rainbow color for letters
//
SpringyLetters.prototype.getRainbowColor = function(r) // r is 0 - 1  - returns a bright color
{
	var a = r*Math.PI*2;
	var r = 128+Math.cos(a)*127;
	var b = 128+Math.cos(a+2*Math.PI/3)*127;
	var g = 128+Math.cos(a+4*Math.PI/3)*127;
	return (r << 16) | (g << 8) | b;
}

// This does most of the set up work.
// It creates a set of letters and sets up the springs
//
// For long titles or tight spaces, you may need to tweak this function
// so that your strings work the way you want them to.
//
SpringyLetters.prototype.setupTitle = function(titleStr)
{
	// Remove existing letters, if any
	for (var i = 0; i < this.nbrLetters; ++i) {
		_root["dot"+i].removeMovieClip();
	}
	this.nbrLetters = 0;
	this.ss.reset();
	
	// Figure out where the string goes and how many MCs we need
	var n = titleStr.length+2;
	var x = this.anchorX;
	var y = this.anchorY;
	var TW = (this.stringWidth-x*2)/n;

	// Setup our first anchor
	var mc = this.addAnchor(x, y, this.firstDotLayer+n+1);
	x += 50;
	var lastMC = mc;	

	// Loop thru the letters and add them, connecting everything with springs
	
	// Initially the letters are strung based on the width of each letter in the embedded font

	for (var i = 0; i < titleStr.length; ++i) 
	{
		// we assign each letter a rainbow color here
		var clr = this.getRainbowColor(i/(.5+titleStr.length-1));

		mc = this.addLetter(x, y, titleStr.charAt(i), clr, this.firstDotLayer+n-i);

		// This adds the spring.  Normally when you add a spring between two points,
		// it is relaxed.  springTautness stretches it a bit by shortening the 'relaxed' length.
		// so it's more like a tight elastic.
		this.ss.addSpring(lastMC, mc, this.springTautness);
		x += mc.letter_txt.textWidth;
		lastMC = mc;
	}

	// Add the final anchor and spring
	x += 30;
	mc = this.addAnchor(x, y, this.firstDotLayer);
	this.ss.addSpring(lastMC, mc, this.springTautness);

	// This stretches the final anchor to it's desired position, causing
	// the letters to careen joyfully due to the whole chain being stretched out.
	mc._x = this.anchorX + this.stringWidth;
	// trace(" nbr dots: " + this.ss.dots.length + ", " + this.ss.hasMC(mc));
}
